@@ -2,6 +2,8 @@
  +----------------------------------------------------------------------+
  | Swoole                                                               |
  +----------------------------------------------------------------------+
+ | Copyright (c) 2012-2018 The Swoole Group                             |
+ +----------------------------------------------------------------------+
  | This source file is subject to version 2.0 of the Apache license,    |
  | that is bundled with this package in the file LICENSE, and is        |
  | available through the world-wide-web at the following url:           |
@@ -27,11 +29,19 @@ enum swAioMode
     SW_AIO_LINUX,
 };
 
-enum
+enum swAioOpcode
 {
     SW_AIO_READ = 0,
     SW_AIO_WRITE = 1,
-    SW_AIO_DNS_LOOKUP = 2,
+    SW_AIO_GETHOSTBYNAME = 2,
+    SW_AIO_GETADDRINFO = 3,
+    SW_AIO_STREAM_GET_LINE = 4,
+};
+
+enum swAioFlag
+{
+    SW_AIO_WRITE_FSYNC = 1u << 1,
+    SW_AIO_EOF         = 1u << 2,
 };
 
 typedef struct _swAio_event
@@ -39,14 +49,19 @@ typedef struct _swAio_event
     int fd;
     int task_id;
     uint8_t type;
+    uint16_t flags;
     off_t offset;
     size_t nbytes;
     void *buf;
     void *req;
     int ret;
     int error;
+    void *object;
+    void (*callback)(struct _swAio_event *event);
 } swAio_event;
 
+typedef void (*swAio_handler)(swAio_event *event);
+
 typedef struct
 {
     uint8_t init;
@@ -56,6 +71,7 @@ typedef struct
     uint16_t current_id;
     swLock lock;
 
+    swAio_handler handlers[SW_AIO_HANDLER_MAX_SIZE];
     void (*destroy)(void);
     void (*callback)(swAio_event *aio_event);
     int (*read)(int fd, void *outbuf, size_t size, off_t offset);
@@ -70,6 +86,7 @@ int swAio_init(void);
 void swAio_free(void);
 int swAioBase_init(int max_aio_events);
 int swAio_dns_lookup(void *hostname, void *ip_addr, size_t size);
+int swAio_dispatch(swAio_event *_event);
 
 #ifdef HAVE_GCC_AIO
 int swAioGcc_init(int max_aio_events);
