@@ -68,6 +68,16 @@ enum swErrorCode
     SW_ERROR_HTTP2_STREAM_ID_TOO_BIG = 3001,
     SW_ERROR_HTTP2_STREAM_NO_HEADER,
 
+    /**
+     * AIO
+     */
+    SW_ERROR_AIO_BAD_REQUEST   = 4001,
+
+    /**
+     * Client
+     */
+    SW_ERROR_CLIENT_NO_CONNECTION = 5001,
+
     SW_ERROR_SOCKS5_UNSUPPORT_VERSION = 7001,
     SW_ERROR_SOCKS5_UNSUPPORT_METHOD,
     SW_ERROR_SOCKS5_AUTH_FAILED,
