@@ -17,6 +17,8 @@
 #ifndef EXT_SWOOLE_PHP7_WRAPPER_H_
 #define EXT_SWOOLE_PHP7_WRAPPER_H_
 
+#include "ext/standard/php_http.h"
+
 #if PHP_MAJOR_VERSION < 7
 typedef zend_rsrc_list_entry zend_resource;
 #define SW_RETURN_STRING                      RETURN_STRING
@@ -57,6 +59,7 @@ static inline int sw_zend_hash_find(HashTable *ht, char *k, int len, void **v)
 #define sw_zend_hash_add                      zend_hash_add
 #define sw_zend_hash_index_update             zend_hash_index_update
 #define sw_call_user_function_ex              call_user_function_ex
+#define sw_zend_register_class_alias          zend_register_class_alias
 
 static sw_inline int sw_call_user_function_fast(zval *function_name, zend_fcall_info_cache *fci_cache, zval **retval_ptr_ptr, uint32_t param_count, zval ***params TSRMLS_DC)
 {
@@ -159,8 +162,7 @@ static inline int SW_Z_TYPE_P(zval *z)
 #define IS_TRUE    1
 inline int SW_Z_TYPE_P(zval *z);
 #define SW_Z_TYPE_PP(z)        SW_Z_TYPE_P(*z)
-
-static inline char* sw_http_build_query(zval *data, zend_size_t *length, smart_str *formstr TSRMLS_DC)
+static sw_inline char* sw_http_build_query(zval *data, zend_size_t *length, smart_str *formstr TSRMLS_DC)
 {
 #if PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION == 3
     if (php_url_encode_hash_ex(HASH_OF(data), formstr, NULL, 0, NULL, 0, NULL, 0, NULL, NULL TSRMLS_CC) == FAILURE)
@@ -183,7 +185,9 @@ static inline char* sw_http_build_query(zval *data, zend_size_t *length, smart_s
     return formstr->c;
 }
 
-#else
+#define sw_get_object_handle(object)        Z_OBJ_HANDLE_P(object)
+
+#else /* PHP Version 7 */
 #define sw_php_var_serialize                php_var_serialize
 typedef size_t zend_size_t;
 #define ZEND_SET_SYMBOL(ht,str,arr)         zval_add_ref(arr); zend_hash_str_update(ht, str, sizeof(str)-1, arr);
@@ -438,7 +442,27 @@ static inline int sw_zend_hash_exists(HashTable *ht, char *k, int len)
     }
 }
 
-static inline char* sw_http_build_query(zval *data, zend_size_t *length, smart_str *formstr TSRMLS_DC)
+static inline int sw_zend_register_class_alias(const char *name, zend_class_entry *ce)
+{
+    int name_len = strlen(name);
+    zend_string *_name;
+    if (name[0] == '\\')
+    {
+        _name = zend_string_init(name, name_len, 1);
+        zend_str_tolower_copy(ZSTR_VAL(_name), name + 1, name_len - 1);
+    }
+    else
+    {
+        _name = zend_string_init(name, strlen(name), 1);
+        zend_str_tolower_copy(ZSTR_VAL(_name), name, name_len);
+    }
+
+    zend_string *_interned_name = zend_new_interned_string(_name);
+
+    return zend_register_class_alias_ex(_interned_name->val, _interned_name->len, ce);
+}
+
+static sw_inline char* sw_http_build_query(zval *data, zend_size_t *length, smart_str *formstr TSRMLS_DC)
 {
     if (php_url_encode_hash_ex(HASH_OF(data), formstr, NULL, 0, NULL, 0, NULL, 0, NULL, NULL, (int) PHP_QUERY_RFC1738) == FAILURE)
     {
@@ -457,6 +481,8 @@ static inline char* sw_http_build_query(zval *data, zend_size_t *length, smart_s
     return formstr->s->val;
 }
 
-#endif
+#define sw_get_object_handle(object)    Z_OBJ_HANDLE(*object)
+
+#endif /* PHP Version */
 
 #endif /* EXT_SWOOLE_PHP7_WRAPPER_H_ */
