@@ -50,7 +50,7 @@
 #include "Client.h"
 #include "async.h"
 
-#define PHP_SWOOLE_VERSION  "2.1.0"
+#define PHP_SWOOLE_VERSION  "2.1.1"
 #define PHP_SWOOLE_CHECK_CALLBACK
 #define PHP_SWOOLE_ENABLE_FASTCALL
 
@@ -256,6 +256,7 @@ PHP_FUNCTION(swoole_get_local_ip);
 PHP_FUNCTION(swoole_get_local_mac);
 PHP_FUNCTION(swoole_unsupport_serialize);
 PHP_FUNCTION(swoole_coroutine_create);
+PHP_FUNCTION(swoole_coroutine_exec);
 
 //---------------------------------------------------------
 //                  swoole_server
@@ -586,9 +587,29 @@ extern ZEND_DECLARE_MODULE_GLOBALS(swoole);
 
 #define SWOOLE_CLASS_ALIAS(name, name_ns) \
     if (SWOOLE_G(use_namespace)) { \
-        zend_register_class_alias(#name, name##_class_entry_ptr);\
+        sw_zend_register_class_alias(#name, name##_class_entry_ptr);\
     } else { \
-        zend_register_class_alias(name_ns, name##_class_entry_ptr);\
+        sw_zend_register_class_alias(name_ns, name##_class_entry_ptr);\
     }
 
+/* PHP 7.3 forward compatibility */
+#ifndef GC_SET_REFCOUNT
+# define GC_SET_REFCOUNT(p, rc) do { \
+		GC_REFCOUNT(p) = rc; \
+	} while (0)
+#endif
+
+#ifndef GC_IS_RECURSIVE
+# define GC_IS_RECURSIVE(p) \
+	(ZEND_HASH_GET_APPLY_COUNT(p) > 1)
+# define GC_PROTECT_RECURSION(p) \
+	ZEND_HASH_INC_APPLY_COUNT(p)
+# define GC_UNPROTECT_RECURSION(p) \
+	ZEND_HASH_DEC_APPLY_COUNT(p)
+#endif
+
+#ifndef ZEND_HASH_APPLY_PROTECTION
+# define ZEND_HASH_APPLY_PROTECTION(p) 1
+#endif
+
 #endif	/* PHP_SWOOLE_H */
