@@ -111,7 +111,7 @@ void* swLinkedList_pop(swLinkedList *ll)
 
 void swLinkedList_remove_node(swLinkedList *ll, swLinkedList_node *remove_node)
 {
-    if (ll->num == 0)
+    if (ll->num == 0 || remove_node == NULL)
     {
         return;
     }
@@ -152,6 +152,29 @@ void swLinkedList_remove_node(swLinkedList *ll, swLinkedList_node *remove_node)
     sw_free(remove_node);
 }
 
+swLinkedList_node* swLinkedList_find(swLinkedList *ll, void *data)
+{
+    if (ll->num == 0)
+    {
+        return NULL;
+    }
+
+    swLinkedList_node *node = ll->head;
+    swLinkedList_node *tmp;
+
+    while (node)
+    {
+        tmp = node->next;
+        if (node->data == data)
+        {
+            return node;
+        }
+        node = tmp;
+    }
+
+    return NULL;
+}
+
 void* swLinkedList_shift(swLinkedList *ll)
 {
     if (ll->head == NULL)
