@@ -262,10 +262,13 @@ static int swWorker_onStreamPackage(swConnection *conn, char *data, uint32_t len
     swServer *serv = SwooleG.serv;
     swEventData *task = (swEventData *) (data + 4);
 
+    serv->last_stream_fd = conn->fd;
+
     swString *package = swWorker_get_buffer(serv, task->info.from_id);
     uint32_t data_length = length - sizeof(task->info) - 4;
     //merge data to package buffer
-    swString_append_ptr(package, data + sizeof(task->info) + 4, data_length);
+    memcpy(package->str, data + sizeof(task->info) + 4, data_length);
+    package->length = data_length;
 
     swWorker_onTask(&serv->factory, task);
 
@@ -286,6 +289,7 @@ int swWorker_onTask(swFactory *factory, swEventData *task)
 #endif
 
     factory->last_from_id = task->info.from_id;
+    serv->last_session_id = task->info.fd;
     swWorker *worker = SwooleWG.worker;
     //worker busy
     worker->status = SW_WORKER_BUSY;
@@ -328,7 +332,8 @@ int swWorker_onTask(swFactory *factory, swEventData *task)
         if (task->info.len > 0)
         {
             //merge data to package buffer
-            swString_append_ptr(package, task->data, task->info.len);
+            memcpy(package->str + package->length, task->data, task->info.len);
+            package->length += task->info.len;
         }
         //package end
         if (task->info.type == SW_EVENT_PACKAGE_END)
@@ -353,12 +358,8 @@ int swWorker_onTask(swFactory *factory, swEventData *task)
         if (package->offset == package->length - sizeof(swDgramPacket))
         {
             worker->request_count++;
-            worker->request_time = SwooleGS->now;
             sw_atomic_fetch_add(&SwooleStats->request_count, 1);
             serv->onPacket(serv, task);
-            worker->request_time = 0;
-            worker->traced = 0;
-            worker->request_count++;
             swString_clear(package);
         }
         break;
@@ -368,7 +369,7 @@ int swWorker_onTask(swFactory *factory, swEventData *task)
         conn = swServer_connection_verify_no_ssl(serv, task->info.fd);
         if (conn && conn->ssl_client_cert.length > 0)
         {
-            sw_free(conn->ssl_client_cert.str);
+            free(conn->ssl_client_cert.str);
             bzero(&conn->ssl_client_cert, sizeof(conn->ssl_client_cert.str));
         }
 #endif
@@ -560,6 +561,13 @@ static void swWorker_stop()
         swReactor_remove_read_event(SwooleG.main_reactor, worker->pipe_worker);
     }
 
+    if (serv->stream_fd > 0)
+    {
+        SwooleG.main_reactor->del(SwooleG.main_reactor, serv->stream_fd);
+        close(serv->stream_fd);
+        serv->stream_fd = 0;
+    }
+
     if (serv->onWorkerStop)
     {
         serv->onWorkerStop(serv, SwooleWG.id);
