@@ -280,6 +280,10 @@ ZEND_END_ARG_INFO()
 ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_coroutine_create, 0, 0, 1)
     ZEND_ARG_INFO(0, func)
 ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_coroutine_exec, 0, 0, 1)
+    ZEND_ARG_INFO(0, command)
+ZEND_END_ARG_INFO()
 #endif
 
 ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_async_exec, 0, 0, 2)
@@ -363,6 +367,7 @@ const zend_function_entry swoole_functions[] =
 #ifdef SW_COROUTINE
     PHP_FE(swoole_async_dns_lookup_coro, arginfo_swoole_async_dns_lookup_coro)
     PHP_FE(swoole_coroutine_create, arginfo_swoole_coroutine_create)
+    PHP_FE(swoole_coroutine_exec, arginfo_swoole_coroutine_exec)
     PHP_FALIAS(go, swoole_coroutine_create, arginfo_swoole_coroutine_create)
 #endif
     /*------other-----*/
@@ -870,6 +875,17 @@ PHP_MINIT_FUNCTION(swoole)
     SWOOLE_DEFINE(ERROR_DATA_LENGTH_TOO_LARGE);
     SWOOLE_DEFINE(ERROR_TASK_PACKAGE_TOO_BIG);
     SWOOLE_DEFINE(ERROR_TASK_DISPATCH_FAIL);
+
+    /**
+     * AIO
+     */
+    SWOOLE_DEFINE(ERROR_AIO_BAD_REQUEST);
+
+    /**
+     * Client
+     */
+    SWOOLE_DEFINE(ERROR_CLIENT_NO_CONNECTION);
+
     SWOOLE_DEFINE(ERROR_HTTP2_STREAM_ID_TOO_BIG);
     SWOOLE_DEFINE(ERROR_HTTP2_STREAM_NO_HEADER);
     SWOOLE_DEFINE(ERROR_SOCKS5_UNSUPPORT_VERSION);
@@ -902,7 +918,7 @@ PHP_MINIT_FUNCTION(swoole)
     }
     else
     {
-        zend_register_class_alias("Co\\Server", swoole_server_class_entry_ptr);
+        sw_zend_register_class_alias("Co\\Server", swoole_server_class_entry_ptr);
     }
 
     zend_declare_property_null(swoole_server_class_entry_ptr, ZEND_STRL("onConnect"), ZEND_ACC_PUBLIC TSRMLS_CC);
@@ -932,7 +948,7 @@ PHP_MINIT_FUNCTION(swoole)
     zend_declare_property_null(swoole_server_class_entry_ptr, ZEND_STRL("ports"), ZEND_ACC_PUBLIC TSRMLS_CC);
     zend_declare_property_long(swoole_server_class_entry_ptr, ZEND_STRL("master_pid"), 0, ZEND_ACC_PUBLIC TSRMLS_CC);
     zend_declare_property_long(swoole_server_class_entry_ptr, ZEND_STRL("manager_pid"), 0, ZEND_ACC_PUBLIC TSRMLS_CC);
-    zend_declare_property_long(swoole_server_class_entry_ptr, ZEND_STRL("worker_id"), 0, ZEND_ACC_PUBLIC TSRMLS_CC);
+    zend_declare_property_long(swoole_server_class_entry_ptr, ZEND_STRL("worker_id"), -1, ZEND_ACC_PUBLIC TSRMLS_CC);
     zend_declare_property_bool(swoole_server_class_entry_ptr, ZEND_STRL("taskworker"), 0, ZEND_ACC_PUBLIC TSRMLS_CC);
     zend_declare_property_long(swoole_server_class_entry_ptr, ZEND_STRL("worker_pid"), 0, ZEND_ACC_PUBLIC TSRMLS_CC);
 
@@ -962,6 +978,12 @@ PHP_MINIT_FUNCTION(swoole)
 
     //swoole init
     swoole_init();
+
+#ifdef SW_COROUTINE
+    memset(&COROG, 0, sizeof(COROG));
+    swReactorCheckPoint = NULL;
+#endif
+
     swoole_server_port_init(module_number TSRMLS_CC);
     swoole_client_init(module_number TSRMLS_CC);
 #ifdef SW_COROUTINE
@@ -1059,6 +1081,9 @@ PHP_MINFO_FUNCTION(swoole)
     php_info_print_table_row(2, "Version", PHP_SWOOLE_VERSION);
     php_info_print_table_row(2, "Author", "tianfeng.han[email: mikan.tenny@gmail.com]");
 
+#ifdef SW_COROUTINE
+    php_info_print_table_row(2, "coroutine", "enabled");
+#endif
 #ifdef HAVE_EPOLL
     php_info_print_table_row(2, "epoll", "enabled");
 #endif
