@@ -123,8 +123,10 @@ void swoole_channel_coro_init(int module_number TSRMLS_DC)
 
     if (SWOOLE_G(use_shortname))
     {
-        zend_register_class_alias("chan", swoole_channel_coro_class_entry_ptr);
+        sw_zend_register_class_alias("chan", swoole_channel_coro_class_entry_ptr);
     }
+
+    zend_declare_property_long(swoole_channel_coro_class_entry_ptr, SW_STRL("capacity")-1, 0, ZEND_ACC_PUBLIC TSRMLS_CC);
 }
 
 static void channel_selector_clear(channel_selector *selector, swLinkedList_node *_node)
@@ -149,7 +151,7 @@ static void channel_selector_onTimeout(swTimer *timer, swTimer_node *tnode)
     ZVAL_BOOL(result, 0);
 
     channel_node *node = tnode->data;
-    channel_selector *selector = tnode->data;
+    channel_selector *selector = node->selector;
 
     zval_ptr_dtor(selector->read_list);
     ZVAL_COPY_VALUE(selector->read_list, &selector->readable);
@@ -170,6 +172,7 @@ static void channel_selector_onTimeout(swTimer *timer, swTimer_node *tnode)
     }
     sw_zval_ptr_dtor(&result);
     efree(selector);
+    efree(node);
 }
 
 static void swoole_channel_onResume(php_context *ctx)
@@ -415,6 +418,8 @@ static PHP_METHOD(swoole_channel_coro, __construct)
     }
     property->closed = 0;
 
+    zend_update_property_long(swoole_channel_coro_class_entry_ptr, getThis(), ZEND_STRL("capacity"), capacity TSRMLS_CC);
+
     swoole_set_property(getThis(), CHANNEL_CORO_PROPERTY_INDEX, property);
 }
 
@@ -434,6 +439,8 @@ static PHP_METHOD(swoole_channel_coro, __destruct)
 
 static PHP_METHOD(swoole_channel_coro, push)
 {
+    coro_check(TSRMLS_C);
+
     swChannel *chan = NULL;
     zval *zdata = NULL;
     int ret;
@@ -483,6 +490,8 @@ static PHP_METHOD(swoole_channel_coro, push)
 
 static PHP_METHOD(swoole_channel_coro, pop)
 {
+    coro_check(TSRMLS_C);
+
     int ret;
     swChannel *chan = swoole_get_object(getThis());
     zval zdata;
@@ -507,7 +516,6 @@ static PHP_METHOD(swoole_channel_coro, pop)
     else
     {
         try_resume_producer_defer(getThis(), property, chan);
-        Z_TRY_DELREF(zdata);
         RETURN_ZVAL(&zdata, 0, NULL);
     }
 }
@@ -544,15 +552,25 @@ static PHP_METHOD(swoole_channel_coro, isFull)
 
 static PHP_METHOD(swoole_channel_coro, stats)
 {
+    channel_coro_property *property = swoole_get_property(getThis(), CHANNEL_CORO_PROPERTY_INDEX);
+
     swChannel *chan = swoole_get_object(getThis());
     array_init(return_value);
 
-    sw_add_assoc_long_ex(return_value, ZEND_STRS("queue_num"), chan->num);
-    sw_add_assoc_long_ex(return_value, ZEND_STRS("queue_bytes"), chan->bytes);
+    sw_add_assoc_long_ex(return_value, ZEND_STRS("consumer_num"), property->consumer_list->num);
+    sw_add_assoc_long_ex(return_value, ZEND_STRS("producer_num"), property->producer_list->num);
+
+    if (chan)
+    {
+        sw_add_assoc_long_ex(return_value, ZEND_STRS("queue_num"), chan->num);
+        sw_add_assoc_long_ex(return_value, ZEND_STRS("queue_bytes"), chan->bytes);
+    }
 }
 
 static PHP_METHOD(swoole_channel_coro, select)
 {
+    coro_check(TSRMLS_C);
+
     zval *read_list, *write_list = NULL, *item;
     zval readable, writable;
     double timeout = 0;
