@@ -77,7 +77,7 @@ void coro_check(TSRMLS_D)
 {
 	if (!COROG.require)
 	{
-        swoole_php_fatal_error(E_ERROR, "coroutine client should use under swoole server in onRequet, onReceive, onConnect callback.");
+        swoole_php_fatal_error(E_ERROR, "must be called in the coroutine.");
 	}
 }
 
@@ -290,7 +290,7 @@ int sw_coro_create(zend_fcall_info_cache *fci_cache, zval **argv, int argc, zval
     {
         zend_execute_ex(call);
         coro_close(TSRMLS_C);
-        swTraceLog(SW_TRACE_COROUTINE, "[CORO_END] Create the %d coro with stack. heap size: %zu\n", COROG.coro_num, zend_memory_usage(0));
+        swTraceLog(SW_TRACE_COROUTINE, "[CORO_END] Create the %d coro with stack. heap size: %zu", COROG.coro_num, zend_memory_usage(0));
         coro_status = CORO_END;
     }
     else
