@@ -99,8 +99,8 @@ void swoole_http2_client_coro_init(int module_number TSRMLS_DC)
 
     if (SWOOLE_G(use_shortname))
     {
-        zend_register_class_alias("Co\\Http2\\Client", swoole_http2_client_coro_class_entry_ptr);
-        zend_register_class_alias("Co\\Http2\\Request", swoole_http2_request_coro_class_entry_ptr);
+        sw_zend_register_class_alias("Co\\Http2\\Client", swoole_http2_client_coro_class_entry_ptr);
+        sw_zend_register_class_alias("Co\\Http2\\Request", swoole_http2_request_coro_class_entry_ptr);
     }
 
     zend_declare_property_long(swoole_http2_client_coro_class_entry_ptr, SW_STRL("type")-1, 0, ZEND_ACC_PUBLIC TSRMLS_CC);
@@ -826,6 +826,12 @@ static PHP_METHOD(swoole_http2_client_coro, __destruct)
     efree(context);
     swoole_set_property(getThis(), HTTP2_CLIENT_CORO_CONTEXT, NULL);
 
+    swClient *cli = swoole_get_object(getThis());
+    if (!cli)
+    {
+        return;
+    }
+
     zval *zobject = getThis();
     zval *retval = NULL;
     sw_zend_call_method_with_0_params(&zobject, swoole_http2_client_coro_class_entry_ptr, NULL, "close", &retval);
