@@ -84,6 +84,7 @@ ZEND_END_ARG_INFO()
 
 ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_mysql_coro_statement_execute, 0, 0, 1)
     ZEND_ARG_INFO(0, params)
+    ZEND_ARG_INFO(0, timeout)
 ZEND_END_ARG_INFO()
 
 static zend_class_entry swoole_mysql_coro_ce;
@@ -150,9 +151,9 @@ void swoole_mysql_coro_init(int module_number TSRMLS_DC)
 
     if (SWOOLE_G(use_shortname))
     {
-        zend_register_class_alias("Co\\MySQL", swoole_mysql_coro_class_entry_ptr);
-        zend_register_class_alias("Co\\MySQL\\Statement", swoole_mysql_coro_statement_class_entry_ptr);
-        zend_register_class_alias("Co\\MySQL\\Exception", swoole_mysql_coro_exception_class_entry_ptr);
+        sw_zend_register_class_alias("Co\\MySQL", swoole_mysql_coro_class_entry_ptr);
+        sw_zend_register_class_alias("Co\\MySQL\\Statement", swoole_mysql_coro_statement_class_entry_ptr);
+        sw_zend_register_class_alias("Co\\MySQL\\Exception", swoole_mysql_coro_exception_class_entry_ptr);
     }
 
     zend_declare_property_string(swoole_mysql_coro_class_entry_ptr, SW_STRL("serverInfo") - 1, "", ZEND_ACC_PRIVATE TSRMLS_CC);
@@ -195,16 +196,16 @@ static int swoole_mysql_coro_close(zval *this)
     _socket->object = NULL;
     _socket->active = 0;
 
-    if (client->cli->timeout_id > 0)
+    if (client->timer)
     {
-        php_swoole_clear_timer_coro(client->cli->timeout_id TSRMLS_CC);
-        client->cli->timeout_id = 0;
+        swTimer_del(&SwooleG.timer, client->timer);
+        client->timer = NULL;
     }
 
     if (client->statement_list)
     {
         swLinkedList_node *node = client->statement_list->head;
-        while(node)
+        while (node)
         {
             mysql_statement *stmt = node->data;
             if (stmt->object)
@@ -617,7 +618,7 @@ static PHP_METHOD(swoole_mysql_coro, query)
 {
     swString sql;
     bzero(&sql, sizeof(sql));
-    double timeout = 0.0;
+    double timeout = -1;
 
     if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|d", &sql.str, &sql.length, &timeout) == FAILURE)
     {
@@ -633,7 +634,8 @@ static PHP_METHOD(swoole_mysql_coro, query)
     mysql_client *client = swoole_get_object(getThis());
     if (!client)
     {
-        swoole_php_fatal_error(E_WARNING, "object is not instanceof swoole_mysql_coro.");
+        SwooleG.error = SW_ERROR_CLIENT_NO_CONNECTION;
+        swoole_php_error(E_WARNING, "object is not instanceof swoole_mysql_coro.");
         RETURN_FALSE;
     }
 
@@ -660,8 +662,8 @@ static PHP_METHOD(swoole_mysql_coro, query)
     php_context *context = swoole_get_property(getThis(), 0);
     if (timeout > 0)
     {
-        if (php_swoole_add_timer_coro((int) (timeout * 1000), client->fd, &client->cli->timeout_id, (void *) context, NULL TSRMLS_CC) == SW_OK
-                && client->defer)
+        client->timer = SwooleG.timer.add(&SwooleG.timer, (int) (timeout * 1000), 0, context, swoole_mysql_coro_onTimeout);
+        if (client->timer && client->defer)
         {
             context->state = SW_CORO_CONTEXT_IN_DELAYED_TIMEOUT_LIST;
         }
@@ -708,8 +710,8 @@ static PHP_METHOD(swoole_mysql_coro, begin)
         php_context *context = swoole_get_property(getThis(), 0);
         if (timeout > 0)
         {
-            if (php_swoole_add_timer_coro((int) (timeout * 1000), client->fd, &client->cli->timeout_id, (void *) context, NULL TSRMLS_CC) == SW_OK
-                    && client->defer)
+            client->timer = SwooleG.timer.add(&SwooleG.timer, (int) (timeout * 1000), 0, context, swoole_mysql_coro_onTimeout);
+            if (client->timer && client->defer)
             {
                 context->state = SW_CORO_CONTEXT_IN_DELAYED_TIMEOUT_LIST;
             }
@@ -760,8 +762,8 @@ static PHP_METHOD(swoole_mysql_coro, commit)
         double timeout = client->connector.timeout;
         if (timeout > 0)
         {
-            if (php_swoole_add_timer_coro((int) (timeout * 1000), client->fd, &client->cli->timeout_id, (void *) context, NULL TSRMLS_CC) == SW_OK
-                    && client->defer)
+            client->timer = SwooleG.timer.add(&SwooleG.timer, (int) (timeout * 1000), 0, context, swoole_mysql_coro_onTimeout);
+            if (client->timer && client->defer)
             {
                 context->state = SW_CORO_CONTEXT_IN_DELAYED_TIMEOUT_LIST;
             }
@@ -811,8 +813,8 @@ static PHP_METHOD(swoole_mysql_coro, rollback)
         double timeout = client->connector.timeout;
         if (timeout > 0)
         {
-            if (php_swoole_add_timer_coro((int) (timeout * 1000), client->fd, &client->cli->timeout_id, (void *) context, NULL TSRMLS_CC) == SW_OK
-                    && client->defer)
+            client->timer = SwooleG.timer.add(&SwooleG.timer, (int) (timeout * 1000), 0, context, swoole_mysql_coro_onTimeout);
+            if (client->timer && client->defer)
             {
                 context->state = SW_CORO_CONTEXT_IN_DELAYED_TIMEOUT_LIST;
             }
@@ -941,10 +943,7 @@ static PHP_METHOD(swoole_mysql_coro, prepare)
         double timeout = client->connector.timeout;
         if (timeout > 0)
         {
-            if (php_swoole_add_timer_coro((int) (timeout * 1000), client->fd, &client->cli->timeout_id, (void *) context, NULL TSRMLS_CC) == SW_OK)
-            {
-                context->state = SW_CORO_CONTEXT_IN_DELAYED_TIMEOUT_LIST;
-            }
+            client->timer = SwooleG.timer.add(&SwooleG.timer, (int) (timeout * 1000), 0, context, swoole_mysql_coro_onTimeout);
         }
         client->cid = get_current_cid();
         coro_save(context);
@@ -955,7 +954,9 @@ static PHP_METHOD(swoole_mysql_coro, prepare)
 static PHP_METHOD(swoole_mysql_coro_statement, execute)
 {
     zval *params;
-    if (zend_parse_parameters(ZEND_NUM_ARGS()TSRMLS_CC, "a", &params) == FAILURE)
+    double timeout = -1;
+
+    if (zend_parse_parameters(ZEND_NUM_ARGS()TSRMLS_CC, "a|d", &params, &timeout) == FAILURE)
     {
         RETURN_FALSE;
     }
@@ -979,11 +980,10 @@ static PHP_METHOD(swoole_mysql_coro_statement, execute)
     }
 
     php_context *context = swoole_get_property(client->object, 0);
-    double timeout = client->connector.timeout;
     if (timeout > 0)
     {
-        if (php_swoole_add_timer_coro((int) (timeout * 1000), client->fd, &client->cli->timeout_id, (void *) context, NULL TSRMLS_CC) == SW_OK
-                && client->defer)
+        client->timer = SwooleG.timer.add(&SwooleG.timer, (int) (timeout * 1000), 0, context, swoole_mysql_coro_onTimeout);
+        if (client->timer && client->defer)
         {
             context->state = SW_CORO_CONTEXT_IN_DELAYED_TIMEOUT_LIST;
         }
@@ -1007,6 +1007,8 @@ static PHP_METHOD(swoole_mysql_coro_statement, __destruct)
     }
     efree(stmt->object);
     stmt->object = NULL;
+    swLinkedList_remove(stmt->client->statement_list, stmt);
+    efree(stmt);
 }
 
 #ifdef SW_USE_MYSQLND
@@ -1067,32 +1069,32 @@ static PHP_METHOD(swoole_mysql_coro, __destruct)
     mysql_client *client = swoole_get_object(getThis());
     if (!client)
     {
-        swoole_php_fatal_error(E_WARNING, "object is not instanceof swoole_mysql_coro.");
-        RETURN_FALSE;
+        return;
     }
-    else if (client->state != SW_MYSQL_STATE_CLOSED && client->cli)
+    if (client->state != SW_MYSQL_STATE_CLOSED && client->cli)
     {
         swoole_mysql_coro_close(getThis());
     }
-	if (client->buffer) {
-		swString_free(client->buffer);
-	}
+    if (client->buffer)
+    {
+        swString_free(client->buffer);
+    }
     efree(client);
     swoole_set_object(getThis(), NULL);
 
     php_context *context = swoole_get_property(getThis(), 0);
     if (!context)
     {
-		return;
+        return;
+    }
+    if (likely(context->state == SW_CORO_CONTEXT_RUNNING))
+    {
+        efree(context);
+    }
+    else
+    {
+        context->state = SW_CORO_CONTEXT_TERM;
     }
-	if (likely(context->state == SW_CORO_CONTEXT_RUNNING))
-	{
-		efree(context);
-	}
-	else
-	{
-		context->state = SW_CORO_CONTEXT_TERM;
-	}
     swoole_set_property(getThis(), 0, NULL);
 }
 
@@ -1129,8 +1131,8 @@ static int swoole_mysql_coro_onError(swReactor *reactor, swEvent *event)
 		client->result = result;
 		return SW_OK;
 	}
-	client->_defer = 0;
-        client->cid = 0;
+    client->_defer = 0;
+    client->cid = 0;
 	php_context *sw_current_context = swoole_get_property(zobject, 0);
 	int ret = coro_resume(sw_current_context, result, &retval);
     sw_zval_free(result);
@@ -1211,16 +1213,17 @@ static void swoole_mysql_coro_onTimeout(swTimer *timer, swTimer_node *tnode)
 
 	if (client->iowait == SW_MYSQL_CORO_STATUS_CLOSED)
 	{
-		zend_update_property_string(swoole_mysql_coro_class_entry_ptr, zobject, ZEND_STRL("error"), "connect timeout." TSRMLS_CC);
+		zend_update_property_string(swoole_mysql_coro_class_entry_ptr, zobject, ZEND_STRL("error"), "connect timeout" TSRMLS_CC);
 	}
 	else
 	{
-		zend_update_property_string(swoole_mysql_coro_class_entry_ptr, zobject, ZEND_STRL("error"), "query timeout." TSRMLS_CC);
+		zend_update_property_string(swoole_mysql_coro_class_entry_ptr, zobject, ZEND_STRL("error"), "query timeout" TSRMLS_CC);
 	}
+
 	zend_update_property_long(swoole_mysql_coro_class_entry_ptr, zobject, ZEND_STRL("errno"), 110 TSRMLS_CC);
 
 	//timeout close conncttion
-	client->cli->timeout_id = 0;
+	client->timer = NULL;
 	client->state = SW_MYSQL_STATE_QUERY;
     swoole_mysql_coro_close(zobject);
 
@@ -1234,7 +1237,8 @@ static void swoole_mysql_coro_onTimeout(swTimer *timer, swTimer_node *tnode)
 
     int ret = coro_resume(ctx, result, &retval);
 
-    if (ret == CORO_END && retval) {
+    if (ret == CORO_END && retval)
+    {
         sw_zval_ptr_dtor(&retval);
     }
 
@@ -1362,10 +1366,11 @@ static int swoole_mysql_coro_onRead(swReactor *reactor, swEvent *event)
     {
         return swoole_mysql_coro_onHandShake(client TSRMLS_CC);
     }
-    if (client->cli->timeout_id > 0)
+
+    if (client->timer)
     {
-        php_swoole_clear_timer_coro(client->cli->timeout_id TSRMLS_CC);
-        client->cli->timeout_id = 0;
+        swTimer_del(&SwooleG.timer, client->timer);
+        client->timer = NULL;
     }
 
     int sock = event->fd;
@@ -1531,6 +1536,7 @@ static int swoole_mysql_coro_onRead(swReactor *reactor, swEvent *event)
             client->_defer = 0;
             client->iowait = SW_MYSQL_CORO_STATUS_READY;
             client->cid = 0;
+
             php_context *sw_current_context = swoole_get_property(zobject, 0);
             ret = coro_resume(sw_current_context, result, &retval);
             if (result)
