@@ -60,4 +60,7 @@ define("MYSQL_SERVER_DB3", "test");
 
 define("TEST_IMAGE", __DIR__ . "/../../examples/test.jpg");
 
-define("IP_BAIDU", "180.97.33.107");
\ No newline at end of file
+define("IP_BAIDU", "180.97.33.107");
+
+define('HTTP_PROXY_HOST', '127.0.0.1');
+define('HTTP_PROXY_PORT', 8888);
\ No newline at end of file
